@@ -39,19 +39,20 @@
 #' mtcars.observed <- rsql$execute_select(select_sql)
 #' mtcars.observed
 #' @importFrom R6 R6Class
+#' @author ken4rab
 #' @export
 RSQL.class <- R6::R6Class("RSQL", public = list(
   #' @field driver driver  name
   driver = NA,
   #' @field db.name database name
   db.name = NA,
-  #' @field db user
+  #' @field user db user
   user = NA,
-  #' @field db user password
+  #' @field password db password
   password = NA,
-  #' @field db host
+  #' @field host db host
   host = NA,
-  #' @field db host port
+  #' @field port db port
   port = NA,
   # regexp
   #'
@@ -69,7 +70,9 @@ RSQL.class <- R6::R6Class("RSQL", public = list(
   #' @field last.query The last query
   last.query = NA,
   #' @field last.rs  The last resultset
-  last.rs = NA,
+  last.rs = NULL,
+  #' @field results.class  Expected class for db results for running dbClearResult
+  results.class = NULL,
   # counters
   #' @field select.counter  An instance select counter
   select.counter = 0,
@@ -81,6 +84,10 @@ RSQL.class <- R6::R6Class("RSQL", public = list(
   delete.counter = 0,
   #' @field command.counter   An instance command counter
   command.counter = 0,
+  #' @field clear.rs.counter   An instance clear.rs.counter
+  clear.rs.counter = 0,
+  #' @field logger is conigured logger for current class
+  logger = NA,
   #' @description
   #' Initializes a connection
   #' @param drv driver name
@@ -102,6 +109,7 @@ RSQL.class <- R6::R6Class("RSQL", public = list(
       user = self$user, password = self$password,
       host = self$host, port = self$port
     )
+    self$logger <- genLogger(self)
     self$valid.conn
   },
   #' @description
@@ -116,10 +124,33 @@ RSQL.class <- R6::R6Class("RSQL", public = list(
         drv = self$driver, dbname = self$db.name,
         user = self$user, password = self$password, host = self$host, port = self$port
       )
+      self$setupResultClassFromDriver()
+    }
+    else{
+      self$clearLastResult()
     }
     self$conn
   },
   #' @description
+  #' Infer ResultsClass from corresponding driver. Implemented for SQLiteDriver & PqConnection
+  #' @return RSQL object
+
+  setupResultClassFromDriver = function(){
+    logger <- getLogger(self)
+    # SQLite
+    if (inherits(self$driver, "SQLiteDriver")){
+      self$results.class <- "SQLiteResult"
+    }
+    # Postgres
+    if (inherits(self$driver, "PqDriver")){
+      self$results.class <- "PqResult"
+    }
+    if (is.null(self$results.class)){
+      logger$warn("Driver Result class not implemented yet", driver = class(self$driver))
+    }
+    self
+  },
+  #' @description
   #' initialize regexp for scraping entities
   #' @param force force setup?
   #' @return regexp for scraping select expressions
@@ -377,16 +408,39 @@ RSQL.class <- R6::R6Class("RSQL", public = list(
   },
   #' @description
   #'
+  #' clearLast Result for avoiding nasty warning
+  clearLastResult = function(){
+    if (!is.null(self$results.class) & !is.null(self$last.rs)){
+      if (inherits(self$last.rs, self$results.class)){
+        dbClearResult(self$last.rs)
+        self$clear.rs.counter <- self$clear.rs.counter + 1
+      }
+      self$last.rs <- NULL
+    }
+  },
+  #' getSummary
+  #' @description
+  #'
+  #' get RSQL summary string
+  getSummary = function(){
+    ret <- paste("RSQL ", class(self$conn),
+                 ". selects: ", self$select.counter,
+                 ". inserts: ", self$insert.counter,
+                 ". updates: ", self$update.counter,
+                 ". delete: ", self$delete.counter,
+                 ". commands: ", self$command.counter,
+                 ". clearRS: ", self$clear.rs.counter, sep = "")
+    ret
+  },
+  #' @description
+  #'
   #' Disconnects the instance from the database
   disconnect = function() {
     if (!is.null(self$conn)) {
-      if (!is.null(self$last.rs)) {
-        # TODO Fix this call
-        # DBI::dbClearResult(self$last.rs)
-        self$last.rs <- NULL
-      }
+      self$clearLastResult()
       DBI::dbDisconnect(self$conn)
       self$conn <- NULL
+      self$results.class <- NULL
     }
   }
 ))
@@ -399,6 +453,7 @@ RSQL.class <- R6::R6Class("RSQL", public = list(
 #' @param password Database password
 #' @param host Database host
 #' @param port Database port
+#' @author ken4rab
 #' @export
 createRSQL <- function(drv, dbname, user = NULL, password = NULL, host = NULL, port = NULL) {
   RSQL.class$new(drv, dbname, user, password, host, port)
@@ -410,6 +465,7 @@ createRSQL <- function(drv, dbname, user = NULL, password = NULL, host = NULL, p
 #' @import lgr
 #' @param sql_insert The SQL String
 #' @param dbconn The Database Connection to run the query against
+#' @author ken4rab
 sql_execute_insert <- function(sql_insert, dbconn) {
   sql_insert <- gsub(",NA", ",NULL", sql_insert)
   sql_insert <- gsub(", NA", ",NULL", sql_insert)
@@ -436,6 +492,7 @@ sql_execute_insert <- function(sql_insert, dbconn) {
 #'
 #' @param sql_update The update SQL
 #' @param dbconn The Database Connection to run the query against
+#' @author ken4rab
 sql_execute_update <- function(sql_update, dbconn = NULL) {
   ret <- DBI::dbSendQuery(dbconn, sql_update)
   ret
@@ -448,6 +505,7 @@ sql_execute_update <- function(sql_update, dbconn = NULL) {
 #'
 #' @param sql_delete The delete SQL
 #' @param dbconn The Database Connection to run the query against
+#' @author ken4rab
 sql_execute_delete <- function(sql_delete, dbconn = NULL) {
   sql_delete <- gsub(",NA", ",NULL", sql_delete)
   sql_delete <- gsub(", NA", ",NULL", sql_delete)
@@ -463,14 +521,15 @@ sql_execute_delete <- function(sql_delete, dbconn = NULL) {
 #' @import DBI
 #' @param sql_select The delete SQL
 #' @param dbconn The Database Connection to run the query against
+#' @author ken4rab
 sql_execute_select <- function(sql_select, dbconn = NULL) {
   # debug sql_select <- 'select price from v_quotes_id_completed where
   # symbol='alua' order by date DESC LIMIT 1' sql_select <- 'select * from
   # v_quotes'
   sql_select <- gsub(",NA", ",NULL", sql_select)
   sql_select <- gsub(", NA", ",NULL", sql_select)
-
   ret <- DBI::dbGetQuery(dbconn, sql_select)
+  #BI:::dbClearResult(ret)
   ret
 }
 
@@ -482,10 +541,12 @@ sql_execute_select <- function(sql_select, dbconn = NULL) {
 #' @param sql_select The SQL select query
 #' @param sql_insert The SQL insert query
 #' @param ... other variables to considered.
+#' @author ken4rab
 sql_execute_get_insert <- function(dbconn, sql_select, sql_insert, ...) {
   ret <- sql_execute_select(sql_select, dbconn = dbconn)
   if (nrow(ret) == 0) {
-    sql_execute_insert(sql_insert, dbconn = dbconn)
+    insert.rs <- sql_execute_insert(sql_insert, dbconn = dbconn)
+    dbClearResult(insert.rs)
     ret <- sql_execute_select(sql_select, dbconn = dbconn)
   }
   ret[1, ]
@@ -495,6 +556,7 @@ sql_execute_get_insert <- function(dbconn, sql_select, sql_insert, ...) {
 #' Determines string type which needs quotes in an SQL statement
 #'
 #' @param text The text to test
+#' @author ken4rab
 needs_quotes <- function(text) {
   class(text) %in% c("Date", "character")
 }
@@ -503,6 +565,7 @@ needs_quotes <- function(text) {
 #'
 #' @param text The text to test
 #' @param quotes_symbols The quotation characters
+#' @author ken4rab
 is_quoted <- function(text, quotes_symbols = "'") {
   ret <- TRUE
   i <- 1
@@ -524,6 +587,7 @@ is_quoted <- function(text, quotes_symbols = "'") {
 #'
 #' @param unquoted.text The unquoted string to stuff quotes from.
 #' @param quote The quoting symbol. Default is '
+#' @author ken4rab
 stuff_quote <- function(unquoted.text, quote = "'") {
   if (is_quoted(unquoted.text)) {
     stop(paste("stuff_quote function cannot be called with quoted text", unquoted.text))
@@ -535,6 +599,7 @@ stuff_quote <- function(unquoted.text, quote = "'") {
 #' Removes the quotes from the string
 #'
 #' @param text The string to remove the quotes from.
+#' @author ken4rab
 dequote <- function(text) {
   substr(text, 2, nchar(text) - 1)
 }
@@ -544,6 +609,7 @@ dequote <- function(text) {
 #'
 #' @param text The string
 #' @param quotes The quotes
+#' @author ken4rab
 re_quote_alt <- function(text, quotes = "'") {
   text <- unlist(text)
   ret <- vapply(text, FUN = function(x) {
@@ -573,6 +639,7 @@ re_quote_alt <- function(text, quotes = "'") {
 #'
 #' @param text The string
 #' @param quotes The quotes
+#' @author ken4rab
 re_quote <- function(text, quotes = "'") {
   text <- as.character(text)
   if (!is.na(text)) {
@@ -595,6 +662,7 @@ re_quote <- function(text, quotes = "'") {
 #' Adds quotes to a string
 #'
 #' @param text The string to quote
+#' @author ken4rab
 #' @export
 add_quotes <- function(text) {
   ret <- sapply(text, FUN = re_quote)
@@ -608,6 +676,7 @@ add_quotes <- function(text) {
 #'
 #' @param text The string to remove quotes from
 #' @param quotes Quote characters
+#' @author ken4rab
 rm_quotes <- function(text, quotes = "'") {
   if (!is.na(text)) {
     unquoted.text <- text
@@ -626,6 +695,7 @@ rm_quotes <- function(text, quotes = "'") {
 #' stuff quote characters in quoted or not quoted df for DSL or DML operations
 #'
 #' @param text.df Data Frame with corresponding values and fields as colnames
+#' @author ken4rab
 stuff_df_quoted <- function(text.df) {
   if (!is.null(text.df)) {
     if (!is.data.frame(text.df)) {
@@ -672,6 +742,7 @@ stuff_df_quoted <- function(text.df) {
 #' Removes quotes from data.frame columns
 #'
 #' @param text.vector The text vector to remove quotes from.
+#' @author ken4rab
 rm_vector_quotes <- function(text.vector) {
   ret <- sapply(text.vector, FUN = rm_quotes)
   names(ret) <- NULL
@@ -682,6 +753,7 @@ rm_vector_quotes <- function(text.vector) {
 #' add_grep_exact_match
 #'
 #' @param text TEST
+#' @author ken4rab
 add_grep_exact_match <- function(text) {
   text <- gsub("(\\^|\\%)", "\\\\\\1", text)
   paste("^", text, "$", sep = "")
@@ -694,6 +766,7 @@ add_grep_exact_match <- function(text) {
 #' @param table The table from which the delete statement will be generated
 #' @param where_fields The fields used in the where section
 #' @param where_values The values used in the where section
+#' @author ken4rab
 sql_gen_delete <- function(table, where_fields = names(where_values), where_values = NULL) {
   where_values.df <- as.data.frame(where_values)
   names(where_values.df) <- where_fields
@@ -715,6 +788,7 @@ sql_gen_delete <- function(table, where_fields = names(where_values), where_valu
 #' @param order_by Order by fields
 #' @param top Retrieve top records
 #' @param distinct it adds a distinct clause to the query.
+#' @author ken4rab
 sql_gen_select <- function(select_fields, table,
                            where_fields = names(where_values),
                            where_values = NULL,
@@ -766,6 +840,7 @@ sql_gen_select <- function(select_fields, table,
 #' @param where_fields The fields used in the where section
 #' @param where_values The values used in the where section
 #' @importFrom utils str
+#' @author ken4rab
 sql_gen_where <- function(where_fields = names(where_values), where_values) {
   check_fields_values(fields = where_fields, values = where_values, min.length = 0)
   ret <- ""
@@ -851,6 +926,7 @@ sql_gen_where <- function(where_fields = names(where_values), where_values) {
 #'
 #' @param where_fields The fields used in the where section
 #' @param where_values The values used in the where section
+#' @author ken4rab
 sql_gen_where_list <- function(where_fields, where_values) {
   sql_where <- ""
   comma.sep <- ", "
@@ -899,6 +975,7 @@ sql_gen_where_list <- function(where_fields, where_values) {
 #'
 #' @param where_fields The fields used in the where section
 #' @param where_values The values used in the where section
+#' @author ken4rab
 sql_gen_where_or <- function(where_fields = names(where_values), where_values) {
   sql_where <- ""
   if (length(where_fields) > 0) {
@@ -937,11 +1014,12 @@ sql_gen_where_or <- function(where_fields = names(where_values), where_values) {
 #' @param table The table to be affected
 #' @param insert_fields The fields to insert
 #' @param values_df The values to insert. Must be defined as data.frame of values
+#' @author ken4rab
 sql_gen_insert <- function(table, values_df, insert_fields = names(values_df)) {
   values.df <- as.data.frame(values_df)
   names(values.df) <- insert_fields
   values.df <- stuff_df_quoted(values.df)
-  if (length(values_df) > 1 & class(values_df) != "data.frame") {
+  if (length(values_df) > 1 & !inherits(values_df, "data.frame")) {
     stop("Values must be defined as data.frames with same size of columns")
   }
   # Converts all factors to strings
@@ -993,6 +1071,7 @@ sql_gen_insert <- function(table, values_df, insert_fields = names(values_df)) {
 #' Replace NA with NULL in sql statement
 #'
 #' @param sql.code code to replace NA with NULL
+#' @author ken4rab
 replaceNAwithNULL <- function(sql.code) {
   sql.code <- gsub("(,( )?)?NA", "\\1NULL", sql.code)
   # sql.code <- gsub(", NA", ", NULL", sql.code)
@@ -1009,6 +1088,7 @@ replaceNAwithNULL <- function(sql.code) {
 #' @param values The values to update
 #' @param where_fields The fields for where statement
 #' @param where_values The values for where statement
+#' @author ken4rab
 sql_gen_update <- function(table, update_fields = names(values), values, where_fields = names(where_values), where_values) {
   check_fields_values(fields = update_fields, values = values, min.length = 1)
   check_fields_values(fields = where_fields, values = where_values, min.length = 0)
@@ -1044,6 +1124,7 @@ sql_gen_update <- function(table, update_fields = names(values), values, where_f
 #' Returns string w/o leading whitespace
 #'
 #' @param x The string
+#' @author ken4rab
 trim_leading <- function(x) sub("^\\s+", "", x)
 
 #' trim_trailing
@@ -1051,12 +1132,14 @@ trim_leading <- function(x) sub("^\\s+", "", x)
 #' Returns string w/o trailing whitespace
 #'
 #' @param x The string
+#' @author ken4rab
 trim_trailing <- function(x) sub("\\s+$", "", x)
 
 
 #' Returns string w/o leading or trailing whitespace
 #'
 #' @param x The string
+#' @author ken4rab
 trim <- function(x) gsub("^\\s+|\\s+$", "", x)
 
 #' rename_col
@@ -1066,15 +1149,17 @@ trim <- function(x) gsub("^\\s+|\\s+$", "", x)
 #' @param df The date.frame
 #' @param name The name of the column
 #' @param replace_name The new name of the column
+#' @author ken4rab
 rename_col <- function(df, name, replace_name) {
   i <- which(names(df) == name)
   names(df)[i] <- replace_name
   df
 }
 
-#' TODO: WHAT DOES THIS DO AGAIN?
+#' cbind_coerced
 #'
 #' @param ... The parameters
+#' @author ken4rab
 cbind_coerced <- function(...) {
   ret <- cbind(...)
   if ("stringsAsFactors" %in% names(ret)) {
@@ -1088,6 +1173,7 @@ cbind_coerced <- function(...) {
 #'
 #' @param dataframe The data.frame
 #' @param columns The columns to check
+#' @author ken4rab
 df_verify <- function(dataframe, columns) {
   ret <- NULL
   dataframe_names <- names(dataframe)
@@ -1114,6 +1200,7 @@ df_verify <- function(dataframe, columns) {
 #' @param fields The fields (Not used. Included for compatibility)
 #' @param values The values (Not used. Included for compatibility)
 #' @param dbconn The database connection
+#' @author ken4rab
 sql_retrieve <- function(table, fields_uk = names(values_uk), values_uk,
                          fields = names(values), values = NULL,
                          field_id = "id", dbconn = NULL) {
@@ -1156,6 +1243,7 @@ sql_retrieve <- function(table, fields_uk = names(values_uk), values_uk,
 #' @param values The values
 #' @param field_id The field of the serial id
 #' @param dbconn The database connection
+#' @author ken4rab
 sql_retrieve_insert <- function(table, fields_uk = names(values_uk), values_uk,
                                 fields = names(values), values = NULL,
                                 field_id = "id", dbconn = NULL) {
@@ -1192,11 +1280,14 @@ sql_retrieve_insert <- function(table, fields_uk = names(values_uk), values_uk,
     lgr$trace("Retrieved", rows = nrow(row))
     if (nrow(row) == 0) {
       lgr$trace(paste("executing", insert_statement))
-      sql_execute_insert(insert_statement, dbconn = dbconn)
+      # TODO solve dbClearResult in case of sql_retrieve_insert
+      #dbClearResult(row)
+      insert.rs <- sql_execute_insert(insert_statement, dbconn = dbconn)
+      dbClearResult(insert.rs)
       row <- sql_execute_select(select_statement, dbconn = dbconn)
     }
-
     ret <- c(ret, as.numeric(row[, field_id]))
+    #dbClearResult(row)
     i <- i + 1
   }
   ret
@@ -1209,6 +1300,7 @@ sql_retrieve_insert <- function(table, fields_uk = names(values_uk), values_uk,
 #' @param recipe TEST
 #' @param indicator_fields TEST
 #' @noRd
+#' @author ken4rab
 sql_gen_joined_query <- function(dw_definition, recipe, indicator_fields) {
   # sql_gen_select <- function(select_fields, table, where_fields='',
   # where_values=NULL,group_by=c()){
@@ -1288,6 +1380,7 @@ sql_gen_joined_query <- function(dw_definition, recipe, indicator_fields) {
 #'
 #' @param where_clause_list The list of params
 #' @import lgr
+#' @author ken4rab
 parse_where_clause <- function(where_clause_list = c()) {
   where_df <- data.frame(
     lhs = character(), comp = character(), rhs = character(),
@@ -1361,14 +1454,16 @@ parse_where_clause <- function(where_clause_list = c()) {
 
 #' Operator IN for multiple columns
 #'
-#' @param x TEST
-#' @param y TEST
+#' @param x vector x
+#' @param y vector y
+#' @author ken4rab
 "%IN%" <- function(x, y) interaction(x) %in% interaction(y)
 
 
 #' Get package directory
 #'
 #' Gets the path of package data.
+#' @author ken4rab
 #' @export
 getPackageDir <- function() {
   home.dir <- find.package("RSQL", lib.loc = NULL, quiet = TRUE)
@@ -1381,6 +1476,7 @@ getPackageDir <- function() {
 
 #' getCarsdbPath
 #' @param copy a boolean that states whether it should be copied to the home directory or not.
+#' @author ken4rab
 #' @export
 getMtcarsdbPath <- function(copy = TRUE) {
   db.filename <- "mtcars.db"
@@ -1401,6 +1497,7 @@ getMtcarsdbPath <- function(copy = TRUE) {
 #' @param fields Fields names to check
 #' @param values values to check
 #' @param min.length for vectors
+#' @author ken4rab
 check_fields_values <- function(fields, values, min.length = 0) {
   stopifnot(length(values) >= min.length)
   stopifnot(length(fields) == length(values))
